#!/usr/bin/perl
#######################
##  AutoRank v2.0.0  ##
###################################################################################
##  rankem.cgi                                                                   ##
##  ----------                                                                   ##
##  This script controls the counting of incoming hits to the list.              ##
###################################################################################

package RANKEM;
use strict;

###################################################################################
#                             DO NOT EDIT THIS FILE                               # 
###################################################################################

$RANKEM::SCRIPT_DATA_DIR = "./sdata";
RANKEM::problem("Could not load the variables file") unless(-e "$RANKEM::SCRIPT_DATA_DIR/variables.dat");
require "$RANKEM::SCRIPT_DATA_DIR/variables.dat";

sub LOCK_SH { 1 }
sub LOCK_EX { 2 }
sub LOCK_NB { 4 }
sub LOCK_UN { 8 }

RANKEM::parse_query();

if($RANKEM::QUERY{'action'} eq "in") {
  &check_time;
  if($RANKEM::QUERY{'id'} eq "rerank") {
    require "functions.cgi";
    FUNCTIONS::print_the_list(FUNCTIONS::load_member_data());
    print "Location: $VARIABLES::HTML_URL/autorank.html\n\n";
    exit;
  }
  &give_hit($RANKEM::QUERY{'id'});
}
else {
  print "Content-type: text/html\n\n";
  print "Malformed access method";
}
exit;

sub parse_query {
  my(@pairs, $pair, $name, $value);
  @pairs = split(/&/, $ENV{'QUERY_STRING'});
  foreach $pair (@pairs) {
    ($name, $value) = split(/=/, $pair);
    $RANKEM::QUERY{$name} = $value;
  }
}

sub give_hit {
  my($id) = shift;
  if(-e "$VARIABLES::MEMBER_DATA_DIR/$id.mdf") {
    open(DATA, "+<$VARIABLES::MEMBER_DATA_DIR/$id.mdf") || RANKEM::problem("Couldn't open the data file for account $id");
    
    unless(flock(DATA, LOCK_SH | LOCK_NB)) {
      unless(flock(DATA, LOCK_SH)) { die "flock: $!\n"; }
    }
    
    my ($in, $out, $totalIn, $inactive, $url, $title, $descrip, $email, $banner, $password) = split(/\|/, <DATA>);
    
    flock(DATA, LOCK_UN);
    
    if($title eq "") {  close(DATA); &repair_data($id);  }
    $in++;
    $totalIn++;
    
    unless(flock(DATA, LOCK_EX | LOCK_NB)) {
      die "flock: $!\n";
      unless(flock(DATA, LOCK_EX)) { die "flock: $!\n"; }
    }
    
    truncate(DATA, 0);
    seek(DATA, 0, 0);
    print DATA "$in|$out|$totalIn|0|$url|$title|$descrip|$email|$banner|$password|";
    flock(DATA, LOCK_UN);
    close(DATA);

    print "Location: $VARIABLES::HTML_URL/autorank.html\n\n";
  }
  else {
    print "Location: $VARIABLES::HTML_URL/autorank.html\n\n";
  } 
}

sub repair_data {
  my $user = shift;
  open(BACKUP, "$VARIABLES::MEMBER_DATA_DIR/$user.bdf") || RANKEM::problem("Couldn't open the data file for account $user");
  my $line = <BACKUP>;
  close(BACKUP);
  
  open(USER, ">$VARIABLES::MEMBER_DATA_DIR/$user.mdf") || RANKEM::problem("Couldn't open the data file for account $user");
  print USER "$line";
  close(USER);
  
  print "Location: $VARIABLES::HTML_URL/autorank.html\n\n";
  exit;
}

sub check_time {
  require "$RANKEM::SCRIPT_DATA_DIR/time.dat";
  my $current_time = time;
 
  my $age = $current_time - $TIME::RERANK;
  my $lastclean = $current_time - $TIME::RESET;
 
  if ($age >= $VARIABLES::RERANK_TIME) {
    open(TIME, ">$RANKEM::SCRIPT_DATA_DIR/time.dat") || RANKEM::problem("Cannot open time data file");
    
    unless(flock(TIME, LOCK_EX | LOCK_NB)) {
      unless(flock(TIME, LOCK_EX)) { die "flock: $!\n"; }
    }
    
    print TIME "package TIME;\n";
    print TIME "\$TIME::RERANK = $current_time;\n";
    print TIME "\$TIME::RESET  = $TIME::RESET;\n";
    print TIME "1;";
    flock(TIME, LOCK_UN);
    close(TIME);
    
    require "functions.cgi";
    FUNCTIONS::print_the_list(FUNCTIONS::load_member_data());
  }

  if ($lastclean >= $VARIABLES::RESET_TIME) {
    open(TIME, ">$RANKEM::SCRIPT_DATA_DIR/time.dat") || RANKEM::problem("Cannot open time data file");
    
    unless(flock(TIME, LOCK_EX | LOCK_NB)) {
      unless(flock(TIME, LOCK_EX)) { die "flock: $!\n"; }
    }
    
    print TIME "package TIME;\n";
    print TIME "\$TIME::RERANK = $TIME::RERANK;\n";
    print TIME "\$TIME::RESET  = $TIME::$current_time;\n";
    print TIME "1;";
    flock(TIME, LOCK_UN);
    close(TIME);    

    RANKEM::reset();
  }
  close(TIME);
}

sub reset {
  my $file;
  opendir(MEMBER_DIR, "$VARIABLES::MEMBER_DATA_DIR");
  my @member_files = sort(grep { m/.*\.mdf/ } readdir(MEMBER_DIR));
  closedir(MEMBER_DIR);  

  for $file (@member_files) {
    open(DATA, "+<$VARIABLES::MEMBER_DATA_DIR/$file") || RANKEM::problem("Cannot open member data file");
    flock(DATA, 2);
    my ($in, $out, $totalIn, $inactive, $url, $title, $descrip, $email, $banner, $password) = split(/\|/, <DATA>);
    $inactive++ unless($in);
    truncate(DATA, 0);
    seek(DATA, 0, 0);
    print DATA "0|0|$totalIn|$inactive|$url|$title|$descrip|$email|$banner|$password|";
    close(DATA);
  }
}

sub get_date {	
  my @time_breakdown = localtime;
  my $time_of_day = "am";
  
  for(0..3) { $time_breakdown[$_] = 0 . $time_breakdown[$_] if(length($time_breakdown[$_]) < 2); } 
  
  $time_breakdown[4]++;
  ($time_breakdown[2] -= 12 and $time_of_day = "pm") if($time_breakdown[2] > 12);  ## Convert to non-military clock
  $time_breakdown[5] += 1900;
  

  return "$time_breakdown[2]:$time_breakdown[1]:$time_breakdown[0]$time_of_day $time_breakdown[4]/$time_breakdown[3]/$time_breakdown[5]";
}

sub problem {
  my $description = shift;
  print "Content-type: text/html\n\n";
  print "$description<br>";
  print "$!";
  exit;
}
